import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * This class has to be the superclass of all actors you want to delay.
 * 
 * The act method of this class is final so you can't use an act method in the subclasses.
 * Everytime the act method is called and there is no delay a method called run will be executed. So you can just change your act method's name into public void run().
 * 
 * The public void run() has to be declared in your class although you don't need it. Otherwhile there will be a compiler error saying that the abstract method run has to be overwriten.
 */
public abstract class DelayableActor extends Actor
{
    protected long delayingTime;
    protected long startingDelay;
    protected long delayingActs;
    protected long executionDelay;
    
    /**
     * This final method executes the run method every time there is no delay (just like an act method).
     */
    public final void act() {
        if (delayingTime == 0 && delayingActs == 0) {
            run();
        }
        else {
            if (delayingTime != 0 && System.currentTimeMillis() - startingDelay >= delayingTime) {
                startingDelay = 0;
                delayingTime = 0;
            }
            if (delayingActs > 0) {
                delayingActs--;
            }
        }
    }
    
    /**
     * This method works as your new act method.
     * It's almost the same as your act method. Only the name is different.
     * 
     * This method has to be overwritten in every subclass.
     */
    public abstract void run();
    
    /**
     * This method is just called from the worldclass if the execution of the game has been stopped (e.g. when the pause button is pressed).
     */
    public void executionStopped() {
        executionDelay = System.currentTimeMillis();
    }
    /**
     * This method is just called from the worldclass if the execution of the game has been started (e.g. when the run button is pressed).
     */
    public void executionStarted() {
        if (executionDelay != 0 && startingDelay != 0) {
            startingDelay += (System.currentTimeMillis() - executionDelay);
            executionDelay = 0;
        }
    }
    
    /**
     * Using this method you can delay the execution of this actor for a time given in milliseconds.
     * 
     * @param delay
     *      The time the actor is paused in milliseconds.
     */
    public void setDelayingTime(long delay) {
        delayingTime = delay;
        startingDelay = System.currentTimeMillis();
        executionDelay = 0;
    }
    /**
     * Using this method you can delay the execution of this actor for a certain number of act cycles.
     * 
     * @param delay
     *      The number of acts this actor is paused.
     */
    public void setDelayingActs(long delay) {
        delayingActs = delay;
    }
    
    /**
     * Get the current time this actor will stay paused.
     * 
     * @return
     *      The current delaying time of this actor.
     */
    public long getDelayingTime() {
        return delayingTime - (System.currentTimeMillis() - startingDelay);
    }
    /**
     * Get the current number of acts this actor will stay paused.
     * 
     * @return
     *      The current number of acts this actor will stay paused.
     */
    public long getDelayingActs() {
        return delayingActs;
    }
}